@@ -7,7 +7,7 @@ use Mojo::SQLite;
 use Sys::Hostname 'hostname';
 use Time::HiRes 'usleep';
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
 has 'sqlite';
 
@@ -32,10 +32,11 @@ sub enqueue {
 
   my $db = $self->sqlite->db;
   return $db->query(
-    q{insert into minion_jobs (args, delayed, priority, queue, task)
-      values (?, (datetime('now', ? || ' seconds')), ?, ?, ?)},
-    {type => SQL_BLOB, value => encode_json($args)}, $options->{delay} // 0,
-    $options->{priority} // 0, $options->{queue} // 'default', $task
+    q{insert into minion_jobs (args, attempts, delayed, priority, queue, task)
+      values (?, ?, (datetime('now', ? || ' seconds')), ?, ?, ?)},
+    {type => SQL_BLOB, value => encode_json($args)}, $options->{attempts} // 1,
+    $options->{delay} // 0, $options->{priority} // 0,
+    $options->{queue} // 'default', $task
   )->last_insert_id;
 }
 
@@ -44,7 +45,7 @@ sub finish_job { shift->_update(0, @_) }
 
 sub job_info {
   my $info = shift->sqlite->db->query(
-    q{select id, args, strftime('%s',created) as created,
+    q{select id, args, attempts, strftime('%s',created) as created,
         strftime('%s',delayed) as delayed,
         strftime('%s',finished) as finished, priority, queue, result,
         strftime('%s',retried) as retried, retries,
@@ -136,7 +137,7 @@ sub retry_job {
   return !!$self->sqlite->db->query(
     q{update minion_jobs
       set finished = null, priority = coalesce(?, priority),
-        queue = coalesce(?, queue), result = null, retried = datetime('now'),
+        queue = coalesce(?, queue), retried = datetime('now'),
         retries = retries + 1, started = null, state = 'inactive', worker = null,
         delayed = (datetime('now', ? || ' seconds'))
       where id = ? and retries = ? and state in ('failed', 'finished')},
@@ -193,7 +194,7 @@ sub _try {
   my $queues_in = join ',', ('?')x@$queues;
   my $tasks_in = join ',', ('?')x@$tasks;
   
-  my $tx = $db->begin('exclusive');
+  my $tx = $db->begin;
   my $res = $db->query(
     qq{select id from minion_jobs
        where delayed <= datetime('now') and queue in ($queues_in)
@@ -210,7 +211,7 @@ sub _try {
   $tx->commit;
   
   my $info = $db->query(
-    'select id, args, retries, task from minion_jobs where id = ?', $job_id
+    'select id, args, attempts, retries, task from minion_jobs where id = ?', $job_id
   )->hash // return undef;
   $info->{args} = decode_json($info->{args});
   
@@ -306,6 +307,12 @@ These fields are currently available:
 
 Job arguments.
 
+=item attempts
+
+  attempts => 25
+
+Number of times performing this job will be attempted, defaults to C<1>.
+
 =item id
 
   id => '10023'
@@ -338,6 +345,12 @@ These options are currently available:
 
 =over 2
 
+=item attempts
+
+  attempts => 25
+
+Number of times performing this job will be attempted, defaults to C<1>.
+
 =item delay
 
   delay => 10
@@ -397,6 +410,12 @@ These fields are currently available:
 
 Job arguments.
 
+=item attempts
+
+  attempts => 25
+
+Number of times performing this job will be attempted, defaults to C<1>.
+
 =item created
 
   created => 784111777
@@ -670,3 +689,6 @@ create table if not exists minion_workers (
 -- 1 down
 drop table if exists minion_jobs;
 drop table if exists minion_workers;
+
+-- 2 up
+alter table minion_jobs add column attempts integer not null default 1;
